"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.continueKeymap = exports.maybeCloseBlockComment = exports.insertNewlineContinueComment = void 0;
const state_1 = require("@codemirror/state");
const language_1 = require("@codemirror/language");
const lang_javascript_1 = require("@codemirror/lang-javascript");
/**
 * This is modeled after the CodeMirror Markdown mode's
 * ability to continue lists and blockquotes. It's meant to be bound
 * to the "Enter" keybinding with Prec.high: there are many cases
 * where this will "fall through", do nothing, and allow the next
 * keybinding to handle.
 */
const insertNewlineContinueComment = ({ state, dispatch, }) => {
    let tree = (0, language_1.syntaxTree)(state), { doc } = state;
    let dont = null;
    let changes = state.changeByRange((range) => {
        // Don't do anything if we're not in JavaScript mode.
        // This should also cover TypeScript, which is just
        // JavaScript with extra configuration.
        if (!range.empty || !lang_javascript_1.javascriptLanguage.isActiveAt(state, range.from))
            return (dont = { range });
        let pos = range.from;
        let line = doc.lineAt(pos);
        const restOfLine = line.text.slice(pos - line.from).trim();
        // TODO: we could do something more sophisticated here.
        // If we're not at the end of the line,
        // do nothing.
        if (restOfLine) {
            return (dont = { range });
        }
        const node = tree.resolveInner(pos, -1);
        if (node.name === "BlockComment") {
            // If this line is the comment ending, do not
            // continue.
            if (line.text.match(/\*\//)) {
                return (dont = { range });
            }
            let indentation = line.text.match(/^([^\*\/]+)([\*|\/])/);
            let indentStr = indentation?.[1] || " ";
            // If continuing from /**,
            // we want an extra space of indentation
            // to match the second *
            if (indentation?.[2] === "/") {
                indentStr = indentStr + " ";
            }
            let insert = state.lineBreak + `${indentStr}* `;
            return {
                range: state_1.EditorSelection.cursor(pos + insert.length),
                changes: { from: line.to, insert: insert },
            };
        }
        return (dont = { range });
    });
    if (dont)
        return false;
    dispatch(state.update(changes, { scrollIntoView: true, userEvent: "input" }));
    return true;
};
exports.insertNewlineContinueComment = insertNewlineContinueComment;
/**
 * When someone has created the final line of a block comment and
 * they hit /, we should _delete_ the space between the * and /
 * for convenience.
 */
const maybeCloseBlockComment = ({ state, dispatch }) => {
    let tree = (0, language_1.syntaxTree)(state), { doc } = state;
    let dont = null;
    let changes = state.changeByRange((range) => {
        // Don't do anything if we're not in JavaScript mode.
        // This should also cover TypeScript, which is just
        // JavaScript with extra configuration.
        if (!range.empty || !lang_javascript_1.javascriptLanguage.isActiveAt(state, range.from))
            return (dont = { range });
        let pos = range.from;
        let line = doc.lineAt(pos);
        const restOfLine = line.text.slice(pos - line.from).trim();
        // TODO: we could do something more sophisticated here.
        // If we're not at the end of the line,
        // do nothing.
        if (restOfLine) {
            return (dont = { range });
        }
        const node = tree.resolveInner(pos, -1);
        if (node.name === "BlockComment") {
            // If this line is the comment ending, do not
            // continue.
            if (line.text.match(/\*\//)) {
                return (dont = { range });
            }
            if (!line.text.match(/^\s+\* $/)) {
                return (dont = { range });
            }
            let insert = "/";
            return {
                range: state_1.EditorSelection.cursor(pos),
                changes: { from: line.to - 1, to: line.to, insert: insert },
            };
        }
        return (dont = { range });
    });
    if (dont)
        return false;
    dispatch(state.update(changes, { scrollIntoView: true, userEvent: "input" }));
    return true;
};
exports.maybeCloseBlockComment = maybeCloseBlockComment;
/**
 * Keymap to use to enable this extra behavior.
 *
 * Example of what you would put in your extensions array.
 *
 * ```ts
 * Prec.high(keymap.of(continueKeymap)),
 * ```
 */
exports.continueKeymap = [
    { key: "Enter", run: exports.insertNewlineContinueComment },
    { key: "/", run: exports.maybeCloseBlockComment },
];
//# sourceMappingURL=index.js.map